// ==============================================================
//                 ORBITER MODULE: CENTURI1
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// CENTURI1.cpp
// Control module for CENTURI1 vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================



#include "orbitersdk.h"
#include "UMmuSDK.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double PB_FUELMASS = 300;
const double PB_ISP = 11e12;
const double PB_MAXMAINTH = 9000;
const double PB_MAXRETROTH = 9000;
const double PB_MAXRCSTH = 900000;

// Interface for derived vessel class: QJBStar
// ==========================================================

class CENTURI1: public VESSEL2 {
public:
    CENTURI1 (OBJHANDLE hObj, int fmodel);
	void clbkSaveState(FILEHANDLE scn);
       void clbkPostCreation (void);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
 	void DefineAnimations(void); 
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkPostStep (double simtt, double simdt, double mjd);
	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
enum DOCKStatus { RDOOR_UP, RDOOR_DOWN, RDOOR_RAISING, RDOOR_LOWERING } BAY_status, DOCK1_status, DOCK2_status, DOCK3_status, DOCK4_status;
    void CENTURI1::SelectCockpitView (int CAM); 
    void RevertDOCK1 (void);
    void RevertDOCK2 (void);
    void RevertDOCK3 (void);
    void RevertDOCK4 (void);
    void RevertBAYDOOR (void);
void Attachships();
void Detachships();
void SetUMMUAirlockPos (void);
    void HUD_Text(const HUDPAINTSPEC* pHPS, HDC hdc);

// UMMU 2.0 DECLARATION
	UMMUCREWMANAGMENT Crew;
	int SelectedUmmuMember;				// for the SDK demo, select the member to eva
	int iActionAreaDemoStep;			// this is just to show one feature of action area.
	void clbkSetClassCaps_UMMu(void);	// our special SetClassCap function just added for more readability

	// The HUD display method variable, see PDF doc
	char cUmmuHudDisplay[255];			// UMmu hud char variable
	double dHudMessageDelay;			// UMmu hud display delay
	char *SendHudMessage(void);			// UMmu hud display function

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	char cAddUMmuToVessel[255];
	void AddUMmuToVessel(BOOL bStartAdding=FALSE);
private:


THRUSTER_HANDLE th_main, th_retro,  th_rcs[4], th_group[2] ;
TEXTMETRIC tm;

ATTACHMENTHANDLE D1;
ATTACHMENTHANDLE D2;
ATTACHMENTHANDLE D3;
ATTACHMENTHANDLE D4;
ATTACHMENTHANDLE D5;
UINT anim_DOCKARM1;
UINT anim_BAY1;
UINT anim_BAY2;
UINT anim_DOCKARM2A;
UINT anim_DOCKARM2B;
UINT anim_DOCKARM2C;

UINT anim_DOCKARM3;
UINT anim_DOCKARM4A;
UINT anim_DOCKARM4B;
UINT anim_DOCKARM4C;
int JohnSoundID;
int CAM;
int  pit, whichone, do_attach, first_time, f8;
double APSel;
double apd;
double DOCK1_proc;
double BAY_proc;
double DOCK2_proc;
double DOCK3_proc;
double DOCK4_proc;
double LIFT_SPEED;
double L_SPEED;
double db;
double MAX_GRAP_DIST;
int i,PADSel;
VECTOR3 LP1_pos;
VECTOR3 LP2_pos;
VECTOR3 LP3_pos;
VECTOR3 LP4_pos;
VECTOR3 LP5_pos;
BEACONLIGHTSPEC APbeacon[5];
VECTOR3 APbeaconpos[5];
double LP1_INT_POS;
double LP2_INT_POS;
double LP3_INT_POS;
double LP4_INT_POS;

};