// ==============================================================
//                 ORBITER MODULE: METAPROBE
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// METAPROBE.cpp
// Control module for METAPROBE vessel class
//

// ==============================================================
#define ORBITER_MODULE
#include "orbitersdk.h"
#include "METAPROBE.h"
#include "OrbiterSoundSDK35.h"

VISHANDLE MainExternalMeshVisual = 0;


void METAPROBE::clbkPostCreation (void)

{
JohnSoundID=ConnectToOrbiterSoundDLL3(GetHandle());
SoundOptionOnOff3(JohnSoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYHOVERTHRUST,FALSE);
SoundOptionOnOff3(JohnSoundID,PLAYATTITUDETHRUST,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(JohnSoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(JohnSoundID,DISPLAYTIMER,FALSE);
}// Constructor
METAPROBE::METAPROBE (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)



{


AIRLOCK_status = AIRLOCK_UP;
AIRLOCK_proc = 0;
LIFT_SPEED =.25;
CAM=0;
	DefineAnimations ();
	
}
void METAPROBE::SelectCockpitView (int CAM)
{

     switch (CAM) {
  case 0:  //ext
          SetCameraDefaultDirection (_V(0,0,1));
          SetCameraOffset (_V(2,3,31));
          break;
 case 1:  //PAD1
         SetCameraDefaultDirection (_V(0,0,1));
          SetCameraOffset (_V(-2,3,31));
          break;
	 }}
////////////////////////////////////////////////////////
// SendHudMessage
////////////////////////////////////////////////////////
char *METAPROBE::SendHudMessage()
{
dHudMessageDelay=15;
return cUmmuHudDisplay;
}
void METAPROBE::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetSize (67);
	SetEmptyMass (4500000.0);
	SetCameraOffset (_V(-2,3,31));
	InitNavRadios (4);
	SetRotDrag (_V(0.2, 0.2, 0.05));
	SetCW (0.35, 0.35, 2, 6);
	SetCrossSections (_V(250,600,38));
	SetPMI (_V(272, 285, 16));
	EnableTransponder (true);



    PROPELLANT_HANDLE TANK;
    TANK = CreatePropellantResource (FUELMASS);

	// ***************** thruster definitions *******************
	th_main = CreateThruster (_V(0,0,-33), _V(0,0,1), 13000000, TANK, ISP);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
		
	th_retro = CreateThruster (_V(0,0,0), _V(0,0,-1), 13000000, TANK, ISP);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);

	double len = 80;
	double d = 2;
	AddExhaust (th_main, len, d, _V(5.725, 0, -33), _V(0, 0, -1));
	AddExhaust (th_main, len, d, _V(0, 0, -33), _V(0, 0, -1));
	AddExhaust (th_main, len, d, _V(-5.725, 0, -33), _V(0, 0, -1));
	double rlen = 8;
	double rd = 0.35;	
THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];

	th_att_rot[0] = CreateThruster (_V(0.0, 0, 22.759), _V(0,-1,0), MAXRCSTH, TANK, ISP);
	th_att_rot[1] = CreateThruster (_V(0.0, 0, -22.759), _V(0,1,0), MAXRCSTH, TANK, ISP);
	th_att_rot[2] = CreateThruster (_V(0.0, 0, 22.759), _V(0,1,0), MAXRCSTH, TANK, ISP);
	th_att_rot[3] = CreateThruster (_V(0.0, 0, -22.759), _V(0,-1,0), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHDOWN);
	CreateThrusterGroup (th_att_rot+2,   2, THGROUP_ATT_PITCHUP);
    //AddExhaust (th_att_rot[1], 0.5, 0.05, _V(-5, -.851, 0), _V(0, -1,0)); //fd
	//AddExhaust (th_att_rot[1], 0.5, 0.05, _V(1, -2.5, -8), _V(0, -1,0)); //fd
	//AddExhaust (th_att_rot[3], 0.5, 0.05, _V(-1, -2.5, -8), _V(0, 1,0)); //fu
    //AddExhaust (th_att_rot[3], 0.5, 0.05, _V(5, -.851, 0), _V(0, 1,0)); //fu

    //AddExhaust (th_att_rot[2], 1, 0.1, _V(1.4, 3.6, -70.3), _V(0, 1,0)); //bd
    //AddExhaust (th_att_rot[2], 1, 0.1, _V(-1.4, 3.5, -70.3), _V(0, 1,0)); //bd
    //AddExhaust (th_att_rot[0], 1, 0.1, _V(-1.4,- 3.5, -70.3), _V(0,-1,0)); //bu	
    //AddExhaust (th_att_rot[0], 1, 0.1, _V(1.4, -3.6, -70.3), _V(0,-1,0)); //bu

	
	th_att_rot[0] = CreateThruster (_V( 22.759,0,0), _V(0,-1,0), MAXRCSTH, TANK, ISP);
	th_att_rot[1] = CreateThruster (_V(-22.759,0,0), _V(0,1,0), MAXRCSTH, TANK, ISP);
	th_att_rot[2] = CreateThruster (_V(22.759,0,0), _V(0,1,0), MAXRCSTH, TANK, ISP);
	th_att_rot[3] = CreateThruster (_V(-22.759,0,0), _V(0,-1,0), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (th_att_rot, 2, THGROUP_ATT_BANKRIGHT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKLEFT);
//	AddExhaust (th_att_rot[1], 1, 0.1, _V(11.3, -0.758, -70.3), _V(0, -1,0)); //fd
  //  AddExhaust (th_att_rot[3], 1, 0.1, _V(11.3, -0.758, -70.3), _V(0,-1,0)); //fu
    //AddExhaust (th_att_rot[1], 1, 0.1, _V(-11.2, 0.758, -70.3), _V(0, 1,0)); //fd
    //AddExhaust (th_att_rot[3], 1, 0.1, _V(-11.2, 0.758, -70.3), _V(0,1,0)); //fu	

	th_att_rot[0] = CreateThruster (_V( 14.6,0,0), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	th_att_rot[1] = CreateThruster (_V(-14.6,0,0), _V(0,0,1), MAXRCSTH, TANK, ISP);
	th_att_rot[2] = CreateThruster (_V( 14.6,0,0), _V(0,0,1), MAXRCSTH, TANK, ISP);
	th_att_rot[3] = CreateThruster (_V(-14.6,0,0), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (th_att_rot, 2, THGROUP_ATT_YAWRIGHT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWLEFT);
	//BACK
    //AddExhaust (th_att_rot[3], 3.5, 0.1, _V(-3.7, 1.291, -80), _V(-1,0,0));
    //AddExhaust (th_att_rot[0], 3.5, 0.1, _V( 3.4, 1.291, -80), _V(1,0,0));

	//Front
    //AddExhaust (th_att_rot[1], 3.5, 0.1, _V(-4.2, 1.5, -10), _V(-1,0,0));
    //AddExhaust (th_att_rot[2], 3.5, 0.1, _V(4.2, 1.5, -10), _V(1,0,0));

	
	th_att_lin[0] = CreateThruster (_V( 22.759,0,0), _V(0,1,0), MAXRCSTH, TANK, ISP);
	th_att_lin[1] = CreateThruster (_V(-22.759,0,0), _V(0,1,0), MAXRCSTH, TANK, ISP);
	th_att_lin[2] = CreateThruster (_V(22.759,0,0), _V(0,-1,0), MAXRCSTH, TANK, ISP);
	th_att_lin[3] = CreateThruster (_V(-22.759,0,0), _V(0,-1,0), MAXRCSTH, TANK, ISP);	
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);
	//AddExhaust (th_att_lin[2], 1, 0.2, _V(5, -.9, 0), _V(0,1,0)); 
    //AddExhaust (th_att_lin[3], 1, 0.2, _V(-5,-.9, 0), _V(0, 1,0)); 
	//AddExhaust (th_att_lin[0], 1, 0.2, _V(5, .83, 0), _V(0, -1,0)); 
    //AddExhaust (th_att_lin[1], 1, 0.2, _V(-5, .83, 0), _V(0, -1,0)); 

	
	
	th_att_lin[0] = CreateThruster (_V(0,0,0), _V(0,0,1), MAXRCSTH, TANK, ISP);
    th_att_lin[1] = CreateThruster (_V(0,0,0), _V(0,0,-1), MAXRCSTH, TANK, ISP);
    CreateThrusterGroup (th_att_lin,   1, THGROUP_ATT_FORWARD);
    CreateThrusterGroup (th_att_lin+1, 1, THGROUP_ATT_BACK);
//	AddExhaust (th_att_lin[1], 1, 0.2, _V(5, 0, -.9), _V(0, 0,-1)); 
// AddExhaust (th_att_lin[0], 1, 0.2, _V(5, 0, 1), _V(0, 0,1)); 
  //  AddExhaust (th_att_lin[1], 1, 0.2, _V(-5, 0, -.9), _V(0, 0,-1)); 
    //AddExhaust (th_att_lin[0], 1, 0.2, _V(-5, 0, 1), _V(0, 0,1)); 


	th_att_lin[0] = CreateThruster (_V(0,0,22.759), _V(1,0,-0.3), MAXRCSTH, TANK, ISP);
	th_att_lin[1] = CreateThruster (_V(0,0,-22.759), _V(1,0,-0.3), MAXRCSTH, TANK, ISP);
	th_att_lin[2] = CreateThruster (_V(0,0,22.759), _V(-1,0,0.3), MAXRCSTH, TANK, ISP);
	th_att_lin[3] = CreateThruster (_V(0,0,-22.759), _V(-1,0,0.3), MAXRCSTH, TANK, ISP);	
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_RIGHT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_LEFT);
 //AddExhaust (th_att_lin[0], 1, 0.2, _V(6, 0, 0), _V(1,0,0)); 
   // AddExhaust (th_att_lin[2], 1, 0.2, _V(-6,0, 0), _V(-1, 0,0)); 
	// ************************************************
	// Call clbkSetClassCaps_UMMU and select a member
	SelectedUmmuMember=0;
	clbkSetClassCaps_UMMu();	
	
// visual specs
// visual specs
SetMeshVisibilityMode (AddMesh (oapiLoadMeshGlobal ("METAPROBE")),  MESHVIS_EXTERNAL ); //Main ship mesh  | MESHVIS_EXTPASS
SetMeshVisibilityMode (AddMesh (oapiLoadMeshGlobal ("EAGLEVC2")),  MESHVIS_COCKPIT ); //Main ship mesh
SetMeshVisibilityMode (AddMesh (oapiLoadMeshGlobal ("EAGLEVC2")),  MESHVIS_VC ); //Main ship mesh






}
// =======================================================
// UMMU Class Caps
// =======================================================
void METAPROBE::clbkSetClassCaps_UMMu(void)
{

	//--------------------------------------------------------------
	// InitUmmu
	// initialisation of Ummu must be done first ! return of 1 mean OK 
	// return of -999 mean Ummu addon is not installed in user's Orbiter, return of -1 
	// mean misc error. If error simply all function will fail silently 
	// and there will be no MMu. It's a good idea to warn the user. 
	// (see function "WarnUserUMMUNotInstalled" below)
	Crew.InitUmmu(GetHandle());	
	//--------------------------------------------------------------

	//--------------------------------------------------------------
	// GetUserUMmuVersion
	// this check wich version of UMmu addon the user have in his orbiter directory.
	// this may be usefull for future version with more function so you can check
	// if user installation support the new functions. Return -1 on error (not installed, version cannot be read)
	// or version number in float value (ie: 2.0="2.0" version etc etc)
	float UMmuVersion=Crew.GetUserUMmuVersion();

	//--------------------------------------------------------------
	// DefineAirLockShape
	// We set the airlock shape and state, this virtual box will be the one
	// into were a Ummu asking to reenter will be taken in account
	// first parameter is airlock door state (OPEN=TRUE) and other are X,X1,Y,Y1 and Z,Z1 
	// min and max coordinate for this virtual box (vessel local coordinate)
	// Pay attention to reenter when landed (if suitable) and make the box large enough so 
	// it's not a pain for the user to find airlock's entry. (See PDF doc)
	Crew.DefineAirLockShape(FALSE,-3,3,-4,3,0,30);			// Airlock open, 2 meter large 4 meter high 6 meter long (vessel local coordinate)

	// SetMembersPositionOrientationOnEVA
	Crew.SetMembersPosRotOnEVA(_V(0,.76,22.6),_V(0,0,0));	// 3 meters in front of ship (z) facing direction of ship (vessel local coordinate)

	//maximum seat available in this ship
	Crew.SetMaxSeatAvailableInShip(2);

	//default crew member used when the ship is spawned by Orbiter's scenerio editor. 
	Crew.AddCrewMember("Paul Roper",41,65,74,"99M");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Frank Craig",27,67,55,"99");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Craig Collins",25,70,45,"99P");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Doug Jackson",45,70,45,"99D" );	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Keith Ford",45,70,45,"99T");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Paul Foster",45,70,45,"99R" );	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Keith Roper",45,70,45,"99T");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Paul Collins",45,70,45,"99R" );	//(for name and id a-z A-Z 0-9 characters only)
	SelectedUmmuMember =0;  // our current selected member


	// The HUD display method variables, see PDF doc
	cUmmuHudDisplay[0] =0;	// Initialisation of UMmu hud char variable
	dHudMessageDelay =0;	// Initialisation of UMmu delay variable
	strcpy(SendHudMessage(),"Welcome aboard ! E=EVA 1,2=select UMmu D=Open/Close airlock S=info M=add crew");

	// The Add mmu without scenery editor variable see PDF doc
	cAddUMmuToVessel[0]=0;
}

void METAPROBE::clbkPostStep(double simt, double simdt, double mjd)

{
//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	// Here the routine that detect if someone entered the ship (eva or transfer)
	// No need to manage anything here all is automatic, crew is added to your ship, ship's weight is updated,
	// and UMmu vessel is automatically deleted from Orbiter. You may just look the return
	// code to see if someone entered and display wathewer message on panel or other.
	// notice it's FPS friendly, function process only 4 time per second not each frame 
	// and return immediately if airlock closed or no Ummu is detected in vincinity. 
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf(SendHudMessage(),"%s \"%s\" aged %i was transfered to our ship",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
		,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf(SendHudMessage(),"%s \"%s\" aged %i entered into our ship",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
		Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}
	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		//int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			//for(I=0;I<Crew.GetCrewTotalNumber();I++)
			//{
		//		Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
		//	}
		//	strcpy(SendHudMessage(),"Oooh no ! Crash - All crew aboard killed");
		}

		// TIPS: the vertical speed is often reset to zero when there is ground contact
		// this may bug somewhat the death of your crew.
		// to have an accurate VertSpeed at touchdown I recommand to record it at very END 
		// of timestep and use this "old" value. The next frame you'll have the vertspeed value 
		// of *last frame* just before the crash (GroundContact). This ensure an accurate 
		// verticalspeed value. (keep this value in your vessel class and don't forget to 
		// initialize it at zero in setclasscap)
	}

//sprintf(oapiDebugString(),"anim %2.2f", AIRLOCK_proc);

	//---------------------------------------------------------------------------
	// WarnUserUMMUNotInstalled - IMPORTANT helper
	// Put here this function and users will be automatically warned if they don't have
	// UMMU installed or if it's outdated. Warning text duration is 20 seconds and text is:
	// [AddonName] require "Universal MMU" ver 2.0 or higher. Download at www.orbiter.dansteph.com (message countdown in seconds)
	// an additonal "your version of UMMU is outdated" is displayed the last 5 seconds if they have version lower than 2.0
	Crew.WarnUserUMMUNotInstalled("METAPROBE");



	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	AddUMmuToVessel();
	






	
if (AIRLOCK_status >= AIRLOCK_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (AIRLOCK_status == AIRLOCK_RAISING) {
			if (AIRLOCK_proc > 0.0) AIRLOCK_proc = max (0.0, AIRLOCK_proc-da);
			else                AIRLOCK_status = AIRLOCK_UP;
		} else {
			if (AIRLOCK_proc < 1.0) AIRLOCK_proc = min (1.0, AIRLOCK_proc+da);
			else                AIRLOCK_status = AIRLOCK_DOWN;
		}
SetAnimation (anim_AIRLOCK, AIRLOCK_proc);

}

}








void METAPROBE::DefineAnimations(void)

{
static UINT DGrp5[1]={124};
   static MGROUP_ROTATE AIRLOCK(0,DGrp5, 1, _V(0,0,22.5), _V(0,0,1), (float)(45*RAD));
  anim_AIRLOCK = CreateAnimation (0);
   AddAnimationComponent  (anim_AIRLOCK, 0, 1, &AIRLOCK); 
   }

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
return new METAPROBE (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
if (vessel) delete (METAPROBE*)vessel;
}

// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int METAPROBE::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)

{
	// only process keydown events
	if (!down) 
		return 0; 

//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	//
	// ADD REALISM: It's your responsabilities also to set ship's control accordingly to crew aboard.
	//				If you want to disable control if no one is aboard have a look at "SetADCtrlMode()"
	//				and "SetAttitudeMode()" functions of Orbiter. To disable thrusters set their fuel 
	//				ressource to NULL.
	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		// we provide feedback to user (You can display a message on panel or wathewer)
		// here below all the return code possible:
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendHudMessage(),"Transfer to docked ship Ok - %s transfered",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendHudMessage(),"EVA OK - %s left the ship",
			Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendHudMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendHudMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendHudMessage(),"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendHudMessage(),"Error, docked ship do not use UMmu 2.0, ask author to add it");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendHudMessage(),"Misc error with UMMU install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA",
		SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),
		Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer"
			", please press \"E\" to EVA",SelectedUmmuMember,
			Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,
			Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the virtual UMMU airlock door on/off
	if(key==OAPI_KEY_D&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{		
	//
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
		// display state

{RevertAIRLOCK();
		

		
		if(Crew.GetAirlockDoorState()==TRUE)
			strcpy(SendHudMessage(),"Airlock is now open");	
		else
			strcpy(SendHudMessage(),"Airlock is now closed");
		}

	

	}

	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S)
	{
		sprintf(SendHudMessage(),"%i souls aboard ship %s, %i seats available",
			Crew.GetCrewTotalNumber(),GetName(),8-Crew.GetCrewTotalNumber());
		return 1;
	}

	//---------------------------------------------------------------------------
	// ADD some Fun, Eject the guy, No check of all return code here to keep listing small and clear.
	// Notice eject function doesn't check airlock state at all.
	// GOOD IDEA: Get the Object's handle after ejection with function "GetObjHandleOfLastEVACrew"
	// and add to it one very small tank, thruster and smoke, then fire thruster (see pilot ejection of DGIV)
	// BAD IDEA: Not testing the handle returned by "GetObjHandleOfLastEVACrew" before using may
	// cause a CTD if by any bad luck the pointer is invalid (handle==NULL)
	if(key==OAPI_KEY_ESCAPE&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(Crew.EjectCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember))==EVA_OK)
			sprintf(SendHudMessage(),"%s EJECTED",Crew.GetLastEvaedCrewName());
		SelectedUmmuMember=0;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	// better idea is to use the new UMMU Id definition
	// look readme.txt in folder "config/UMMUIdConfig"
	if(key==OAPI_KEY_C)
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendHudMessage(),"Mesh changed");
		return 1;
	}

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	if(key==OAPI_KEY_M&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		AddUMmuToVessel(TRUE);
	}
 

if(key==OAPI_KEY_V)
{
    CAM = CAM + 1;
    if(CAM > 1) CAM = 0;
        SelectCockpitView(CAM);
        return 1;
}



 

 
		return 0;
}
// ====================================================================
// clbkVisualCreated used to display UMMU initialisation message 
// because oapiDebugString() doesn't work in clbkSetClassCap
// ====================================================================
void METAPROBE::clbkVisualCreated (VISHANDLE vis, int refcount)
{   
	MainExternalMeshVisual = GetMesh(vis,0);
}
// ==============================================================
// Visual destroyed
// ==============================================================
void METAPROBE::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
	MainExternalMeshVisual = 0;	
}

	
void METAPROBE::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// UMmu display messages
	if(dHudMessageDelay>0)
	{
		TextOut (hDC,5,hps->H/60*15,cUmmuHudDisplay,strlen(cUmmuHudDisplay));
		dHudMessageDelay-=oapiGetSimStep();
		if(dHudMessageDelay<0)
			dHudMessageDelay=0;
	}
}
void METAPROBE::clbkSaveState(FILEHANDLE scn)
{

	// ORBITER, default vessel parameters
	SaveDefaultState (scn);

	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios(scn);
}
void METAPROBE::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;

		// ORBITER, unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, status);
	}

	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu airlock state and set it accordingly
	// to your airlock. IE: call here "SetAirlockDoorState" TRUE of FALSE depend
	// of your animated airlock
}
bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr=(char*)data;	if(*cPtr==2){*cPtr=3;strcpy(cPtr+2,str);}
	else if(*cPtr==4){*cPtr=5;strcpy(cPtr+42,str);}
	else if(*cPtr==6){*cPtr=7;strcpy(cPtr+82,str);}return true;
}
void METAPROBE::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding==FALSE&&cAddUMmuToVessel[0]==0)
		return;
	if(bStartAdding==TRUE){
		int salut=sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel,0,sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0]=1;
	}
	else if(cAddUMmuToVessel[0]==1){
		cAddUMmuToVessel[0]=2;
		oapiOpenInputBox ("Enter new crew's name (or escape)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==3){
		cAddUMmuToVessel[0]=4;
		oapiOpenInputBox ("Enter crew's age",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==5){
		cAddUMmuToVessel[0]=6;
		oapiOpenInputBox ("Enter function (Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==7){
		cAddUMmuToVessel[0]=0;
		int Age=max(5,min(100,atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2],Age,70,70,&cAddUMmuToVessel[82])==TRUE){
			sprintf(SendHudMessage(),"Crew \"%s\" aged %i added to vessel",&cAddUMmuToVessel[2],Age);
		}
		else{
			strcpy(SendHudMessage(),"ERROR: Crew not added (vessel full?)");
		}
	}
}
// END of  "Allow user to add crew to your ship without scenery editor"
//-------------------------------------------------------------------------






void METAPROBE::RevertAIRLOCK (void)
{
	AIRLOCK_status = ((AIRLOCK_status == AIRLOCK_UP || AIRLOCK_status == AIRLOCK_RAISING) ?
		AIRLOCK_LOWERING : AIRLOCK_RAISING);
}
