// ==============================================================
//                 ORBITER MODULE: METAPROBE
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// METAPROBE.cpp
// Control module for METAPROBE vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================



#include "orbitersdk.h"
#include "UMmuSDK.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double FUELMASS = 500000;
const double ISP = 11e12;
const double MAXMAINTH = 13000000;
const double MAXRETROTH = 13000000;
const double MAXRCSTH = 166666;

// Interface for derived vessel class: QJBStar
// ==========================================================

class METAPROBE: public VESSEL2 {
public:
    METAPROBE (OBJHANDLE hObj, int fmodel);
	void clbkSaveState(FILEHANDLE scn);
       void clbkPostCreation (void);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
 	void DefineAnimations(void); 
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkPostStep (double simtt, double simdt, double mjd);
	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
enum AIRLOCKStatus { AIRLOCK_UP, AIRLOCK_DOWN, AIRLOCK_RAISING, AIRLOCK_LOWERING } AIRLOCK_status;void RevertAIRLOCKOUTER (void);
    void RevertAIRLOCK (void);
void METAPROBE::SelectCockpitView (int CAM); 
void SetUMMUAirlockPos (void);
// UMMU 2.0 DECLARATION
	UMMUCREWMANAGMENT Crew;
	int SelectedUmmuMember;				// for the SDK demo, select the member to eva
	int iActionAreaDemoStep;			// this is just to show one feature of action area.
	void clbkSetClassCaps_UMMu(void);	// our special SetClassCap function just added for more readability

	// The HUD display method variable, see PDF doc
	char cUmmuHudDisplay[255];			// UMmu hud char variable
	double dHudMessageDelay;			// UMmu hud display delay
	char *SendHudMessage(void);			// UMmu hud display function

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	char cAddUMmuToVessel[255];
	void AddUMmuToVessel(BOOL bStartAdding=FALSE);
private:


THRUSTER_HANDLE th_main, th_retro,  th_rcs[4], th_group[2] ;
UINT anim_AIRLOCK;

int JohnSoundID;
double AIRLOCK_proc,LIFT_SPEED;
int  CAM;
};